-- Example of ESF Arrays showing sharing and deletion

module Main where

import Control.Monad.IO.Class
import Control.Monad.Trans.State.Strict
import Data.Array.ESFA.Spec
import Data.Array.ESFA.SimArch

main :: IO ()
main = runtree 4 () script

showState =
  do esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     stats <- esfmAnalyzeMachineState
     liftIO (putStrLn (show stats))
     liftIO (putStrLn (take 75 (repeat '-')))

script :: StESFA () Int ()
script =
  do liftIO (putStrLn "ESFA Example 2")
     esfmInitialize
     esfmSetOption EsfmTraceOperation

     Just a1 <- esfmUpdate "a1" esfmEmpty (100, mkInt 101)
     showState
-- a1 = 100-101

     Just a2 <- esfmUpdate "a2" a1 (200, mkInt 201)
     showState
-- a2 = 200-201 100-101 

     Just a3 <- esfmUpdate "a3" a1 (300, mkInt 301)
     showState
-- a3 = 300-301 100-101

     Just a4 <- esfmUpdate "a4" a2 (400, mkInt 401)
     showState
-- a4 = 400-401 200-201 100-101 

     Just a5 <- esfmUpdate "a5" a2 (200, mkInt 501)
     Just temp <- esfmLookup "a2" a2 200
     liftIO (putStrLn ("a2[200] = " ++ show temp)) -- expect 201
     Just temp <- esfmLookup "a5" a5 200
     liftIO (putStrLn ("a5[200] = " ++ show temp)) -- expect 501
     showState
-- a5 = 500-501 200-201 100-101 

     Just a6 <- esfmUpdate "a6" a3 (600, mkInt 601)
     showState
-- a6 = 600-601 300-301 100-101

     Just x1 <- esfmLookup "a4" a4 400   -- expect 401
     Just x2 <- esfmLookup "a5" a5 100   -- expect 101
     Just x3 <- esfmLookup "a5" a5 200   -- expect 201
     Just x4 <- esfmLookup "a6" a6 300   -- expect 301
     liftIO (putStrLn (show (map getInt [x1,x2,x3,x4])))

     esfmDelete a3
     showState

     esfmDelete a6
     showState




mkInt :: Int -> EsfEltVal Int
mkInt n = EsfUserOrdVal n
getInt :: EsfEltVal Int -> Int
getInt (EsfUserOrdVal n) = n
getInt _ = error "element should be EsfUserOrdVal but isn't"
