-- Example of ESF Arrays showing sharing of elements
-- This is the example that appeared in the PARLE'93 paper

module Main where

import Control.Monad.IO.Class
import Control.Monad.Trans.State.Strict
import Data.Array.ESFA.Spec
import Data.Array.ESFA.SimArch

main :: IO ()
main = runtree 4 () script

script :: StESFA () Int ()
script =
  do liftIO (putStrLn "Test example from PARLE 93")
     esfmInitialize

     Just a1 <- esfmUpdate "a1" esfmEmpty (100, mkInt 101)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

     Just a2 <- esfmUpdate "a2" a1 (200, mkInt 201)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

     Just a3 <- esfmUpdate "a3" a1 (300, mkInt 301)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

     Just a4 <- esfmUpdate "a4" a2 (400, mkInt 401)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

     Just a5 <- esfmUpdate "a5" a2 (500, mkInt 501)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

     Just a6 <- esfmUpdate "a6" a3 (600, mkInt 601)
     esfmPrintState
     esfmPrintMachineArrays
     esfmPrintSpecArrays
     liftIO (putStrLn (take 75 (repeat '-')))

-- a1 = 100-101
-- a2 = 200-201 100-101 
-- a3 = 300-301 100-101
-- a4 = 400-401 200-201 100-101 
-- a5 = 500-501 200-201 100-101 
-- a6 = 600-601 300-301 100-101

     Just x1 <- esfmLookup "a4" a4 400   -- expect 401
     Just x2 <- esfmLookup "a5" a5 500   -- expect 501
     Just x3 <- esfmLookup "a5" a5 200   -- expect 201
     Just x4 <- esfmLookup "a5" a5 100   -- expect 101
     liftIO (putStrLn (show (map getInt [x1,x2,x3,x4])))
     esfmPrintState



mkInt :: Int -> EsfEltVal Int
mkInt n = EsfUserOrdVal n
getInt :: EsfEltVal Int -> Int
getInt (EsfUserOrdVal n) = n
getInt _ = error "element should be EsfUserOrdVal but isn't"
